package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.Client;
import cn.wise.sc.cement.business.entity.Project;
import cn.wise.sc.cement.business.exception.BusinessExceptionEnum;
import cn.wise.sc.cement.business.mapper.ClientMapper;
import cn.wise.sc.cement.business.mapper.ProjectMapper;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.query.ClientQuery;
import cn.wise.sc.cement.business.model.vo.ClientVo;
import cn.wise.sc.cement.business.service.IClientService;
import cn.wise.sc.cement.business.util.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-08-07
 */
@Service
public class ClientServiceImpl extends ServiceImpl<ClientMapper, Client> implements IClientService {

	private static final Logger log = LoggerFactory.getLogger("ClientServiceImpl");

    final
    ProjectMapper projectMapper;
	@Resource
	private ClientMapper clientMapper;



	public ClientServiceImpl(ProjectMapper projectMapper) {
        this.projectMapper = projectMapper;
    }


    @Override
    public BaseResponse<IPage<ClientVo>> getPage(PageQuery pageQuery, Integer status, String region, String principal) {

        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.orderByDesc("id");
        if (status != null) {
            qw.eq("status", status);
        }
        if (StrUtil.isNotBlank(region)) {
            qw.like("region", region);
        }
        if (StrUtil.isNotBlank(principal)) {
            qw.like("principal", principal);
        }
        IPage<Client> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        this.page(page);
        IPage<ClientVo> rts = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        //关联项目名
        List<Client> records = page.getRecords();
        if (records == null) {
            return BaseResponse.okData(rts);
        }
        List<Integer> clientIds = records.stream().map(Client::getId).collect(Collectors.toList());
        BusinessExceptionEnum.COLLECTION_NOT_ILLEGAL.assertCollectionNotILLEGAL(clientIds);
        QueryWrapper<Project> qwProject = new QueryWrapper<>();
        qwProject.in("client_id",clientIds);
        List<Project> projects = projectMapper.selectList(qwProject);

        List<ClientVo> rtsRe = new ArrayList<>(page.getRecords().size());
        for (Client client : page.getRecords()) {
            ClientVo clientVo = client.toVo();
            List<String> projectNames = new ArrayList<>();
            projects.forEach(arg->{
                if (arg.getClientId().intValue() == client.getId()){
                    projectNames.add(arg.getName());
                }
            });
            clientVo.setProjects(projectNames);
            rtsRe.add(clientVo);
        }
        rts.setRecords(rtsRe);
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<List<ClientVo>> getList() {
        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.eq("status", 1);
        List<Client> list = this.list(qw);
        List<ClientVo> rts = new ArrayList<>(list.size());
        list.forEach(arg -> {
            ClientVo clientVo = new ClientVo();
            clientVo.setId(arg.getId());
            clientVo.setName(arg.getName());
            rts.add(clientVo);
        });
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<ClientVo> create(final ClientQuery clientQuery) {
        //判断名字是否重复
        if (clientIsExisting(clientQuery.getName())) {
            return BaseResponse.errorMsg("当前单位名已存在!");
        }
        Client crate = Client.crate(clientQuery);
        this.save(crate);
        return BaseResponse.okData(crate.toVo());
    }

    @Override
    public BaseResponse<ClientVo> update(final ClientQuery clientQuery) {
        //判断名字是否重复
        if (clientIsExisting(clientQuery.getName())) {
            return BaseResponse.errorMsg("当前单位名已存在!");
        }
        Client update = Client.crate(clientQuery);
        update.setId(clientQuery.getId());
        this.updateById(update);
        return BaseResponse.okData(update.toVo());
    }

    @Override
    public BaseResponse<Boolean> status(Integer status, Integer id) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(status);
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(id);

        Client update = new Client();
        update.setId(id);
        update.setStatus(status == 1);

        return BaseResponse.okData(this.updateById(update));
    }


    /**
     * 判断单位是否重复
     *
     * @param name 单位名
     * @return bool
     */
    private boolean clientIsExisting(String name) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(name);
        //判断名字是否重复
        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.eq("name", name);
        qw.eq("status", 1);
        int count = this.count(qw);
        return count > 0;
    }

	@Override
    public void export(Integer status, String region, String principal, String fileName, HttpServletResponse response){
	    Map<String, Object> params = new HashMap<>();
	    params.put("status", status);
	    params.put("region", region);
	    params.put("principal", principal);
	    List<Map<String, Object>> list= clientMapper.exportList(params);

	    if (!CollectionUtils.isEmpty(list)) {
		    Map<String, Object> map = list.get(0);
		    String[] headers = new String[map.size()];
		    headers[0] = "序号";
		    headers[1] = "单位名称";
		    headers[2] = "联系人";
		    headers[3] = "联系电话";
		    headers[4] = "所在地区";
		    headers[5] = "项目数";
		    headers[6] = "状态";

		    List<Object[]> datas = new ArrayList<>(list.size());
		    for (Map<String, Object> m : list) {
			    Object[] objects = new Object[headers.length];
			    for (int j = 0; j < headers.length; j++) {
				    String obj = m.get(headers[j]).toString();
				    if(j==0){
					    obj = obj.split("\\.")[0];
				    }
				    objects[j] = obj;
			    }

			    datas.add(objects);
		    }
		    ExcelUtil.excelExport(
				    fileName == null || fileName.trim().length() <= 0 ? "委托单位": fileName, headers,
				    datas, response);
	    }
    }




}
