package cn.wise.sc.cement.business.service.impl;


import cn.wise.sc.cement.business.entity.Client;
import cn.wise.sc.cement.business.entity.Project;
import cn.wise.sc.cement.business.exception.BusinessExceptionEnum;
import cn.wise.sc.cement.business.mapper.ProjectMapper;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.query.ProjectQuery;
import cn.wise.sc.cement.business.model.vo.ProjectVo;
import cn.wise.sc.cement.business.service.IClientService;
import cn.wise.sc.cement.business.service.IProjectService;
import cn.wise.sc.cement.business.util.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-08-07
 */
@Service
public class ProjectServiceImpl extends ServiceImpl<ProjectMapper, Project> implements IProjectService {

	private static final Logger log = LoggerFactory.getLogger("ProjectServiceImpl");

	final
	IClientService iClientService;

	@Resource
	private ProjectMapper projectMapper;

    public ProjectServiceImpl(IClientService iClientService) {
        this.iClientService = iClientService;
    }

    @Override
    public BaseResponse<IPage<ProjectVo>> getPage(final PageQuery pageQuery,
                                                  final Integer status,
                                                  final Integer clientId) {

        QueryWrapper<Project> qw = new QueryWrapper<>();
        qw.orderByDesc("id");
        if (status != null) {
            qw.eq("status", status);
        }
        if (clientId != null && clientId > 0) {
            qw.like("client_id", clientId);
        }
        IPage<Project> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        this.page(page);
        //Client_id的委托单位列表
        List<Project> records = page.getRecords();
        List<Integer> clientIds = records.stream().map(Project::getClientId).collect(Collectors.toList());
        QueryWrapper<Client> qwClient = new QueryWrapper<>();
        qwClient.in("client_id", clientIds);
        List<Client> clients = iClientService.list(qwClient);
        BusinessExceptionEnum.COLLECTION_NOT_ILLEGAL.assertCollectionNotILLEGAL(clients);

        IPage<ProjectVo> rts = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        List<ProjectVo> rtsRe = new ArrayList<>(page.getRecords().size());
        for (Project project : page.getRecords()) {
            ProjectVo projectVo = project.toVo();
            //Client_id转ClientName
            clients.forEach(arg -> {
                if (arg.getId().intValue() == project.getClientId()) {
                    projectVo.setClientName(arg.getName());
                }
            });
            rtsRe.add(projectVo);
        }
        rts.setRecords(rtsRe);
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<List<ProjectVo>> getList() {
        QueryWrapper<Project> qw = new QueryWrapper<>();
        qw.eq("status", 1);
        List<Project> list = this.list(qw);
        List<ProjectVo> rts = new ArrayList<>(list.size());
        list.forEach(arg -> {
            ProjectVo modelVo = new ProjectVo();
            modelVo.setId(arg.getId());
            modelVo.setName(arg.getName());
            rts.add(modelVo);
        });
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<ProjectVo> create(ProjectQuery query) {
        if (projectIsExisting(query.getCode(), query.getClientId())) {
            return BaseResponse.errorMsg("项目编号已存在!");
        }
        Project crate = Project.crate(query);
        this.save(crate);
        return BaseResponse.okData(crate.toVo());
    }

    @Override
    public BaseResponse<ProjectVo> update(ProjectQuery query) {
        //判断名字是否重复
        if (projectIsExisting(query.getName(), query.getClientId())) {
            return BaseResponse.errorMsg("项目编号已存在!");
        }
        Project update = Project.crate(query);
        update.setId(query.getId());
        this.updateById(update);
        return BaseResponse.okData(update.toVo());
    }

    @Override
    public BaseResponse<Boolean> status(Integer status, Integer id) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(status);
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(id);

        Project update = new Project();
        update.setId(id);
        update.setStatus(status == 1);

        return BaseResponse.okData(this.updateById(update));
    }

    private boolean projectIsExisting(String code, Integer clientId) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(code);
        //判断名字是否重复
        QueryWrapper<Project> qw = new QueryWrapper<>();
        qw.eq("code", code);
        qw.eq("status", 1);
        int count = this.count(qw);
        //判断单位是否存在可用
        if (clientId != null) {
            QueryWrapper<Client> isExistingClient = new QueryWrapper<>();
            isExistingClient.eq("status", 1);
            isExistingClient.eq("id", clientId);
            count += iClientService.count(isExistingClient);
        }
        return count > 0;
    }

	@Override
	public void export(String code, Integer clientId, String name, String fileName, HttpServletResponse response){
		Map<String, Object> params = new HashMap<>();
		params.put("code", code);
		params.put("clientId", clientId);
		params.put("name", name);
		List<Map<String, Object>> list= projectMapper.exportList(params);

		if (!CollectionUtils.isEmpty(list)) {
			Map<String, Object> map = list.get(0);
			String[] headers = new String[map.size()];
			headers[0] = "序号";
			headers[1] = "项目名称";
			headers[2] = "项目编号";
			headers[3] = "所属单位";

			List<Object[]> datas = new ArrayList<>(list.size());
			for (Map<String, Object> m : list) {
				Object[] objects = new Object[headers.length];
				for (int j = 0; j < headers.length; j++) {
					String obj = m.get(headers[j]).toString();
					if(j==0){
						obj = obj.split("\\.")[0];
					}
					objects[j] = obj;
				}
				datas.add(objects);
			}
			ExcelUtil.excelExport(
					fileName == null || fileName.trim().length() <= 0 ? "项目": fileName, headers,
					datas, response);
		}
	}








}
